'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('./fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _pipeStreams = require('./pipe-streams');

var _pipeStreams2 = _interopRequireDefault(_pipeStreams);

var _formatSize = require('./format-size');

var _prompt = require('./prompt');

var _prompt2 = _interopRequireDefault(_prompt);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class DatCp {

  constructor(dat, options) {
    this.dat = dat;
    this.options = options;
    this.files = 0;
    this.totalSize = 0;
  }

  upload(paths) {
    var _this = this;

    return _asyncToGenerator(function* () {
      yield _this.ensurePathsValid(paths);

      if (_this.options.dryRun) {
        _this.printTotal();
        process.exit(0);
      }

      _logger2.default.debug('Creating metadata for files:');
      for (const path of paths) {
        yield _this.uploadPath(path, '');
      }

      _this.printTotal();
    })();
  }

  ensurePathsValid(paths) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      _this2.count = 0;

      for (const path of paths) {
        yield _this2.ensurePathValid(path);
      }

      if (!_this2.files) {
        _logger2.default.error(`No files to copy.`);
        process.exit(1);
      }
    })();
  }

  ensurePathValid(path) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      let stats;

      try {
        stats = yield _fs2.default.lstat(path);
      } catch (err) {
        _logger2.default.error(`${path}: No such file or directory.`);
        process.exit(1);
      }

      if (stats.isFile()) {
        _this3.countPath(path, stats);
        return;
      }

      if (stats.isDirectory()) {
        return yield _this3.ensureDirValid(path, stats);
      }

      _logger2.default.warn(`${path}: Not a file or directory (not copied).`);
      return;
    })();
  }

  ensureDirValid(path, stats) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      if (!_this4.options.recursive) {
        _logger2.default.warn(`${path}: Is a directory (not copied).`);
        return;
      }

      if (path[path.length - 1] !== '/') {
        _this4.countPath(path, stats);
      }

      const dirPaths = yield _fs2.default.readdir(path);

      for (const dirPath of dirPaths) {
        yield _this4.ensurePathValid(_path2.default.join(path, dirPath));
      }
    })();
  }

  uploadPath(path, datPath) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      const stats = yield _fs2.default.lstat(path);

      if (stats.isFile()) {
        yield _this5.uploadFile(path, datPath);
      } else if (stats.isDirectory()) {
        yield _this5.uploadDir(path, datPath);
      }
    })();
  }

  uploadFile(path, datPath) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      datPath = _path2.default.join(datPath, _path2.default.parse(path).base);
      const stats = yield _fs2.default.lstat(path);
      const filesize = stats.size || 1;

      const readStream = _fs2.default.createReadStream(path);
      const writeStream = _this6.dat.archive.createWriteStream(datPath, { path });

      yield (0, _pipeStreams2.default)(readStream, writeStream, filesize, datPath);
    })();
  }

  uploadDir(path, datPath) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      if (!_this7.options.recursive) {
        return;
      }

      // If a source dir ends with `/`, copy its contents, not the dir itself
      if (path[path.length - 1] !== '/') {
        datPath = _path2.default.join(datPath, _path2.default.parse(path).base);
        yield _this7.mkdir(datPath);
      }

      const dirPaths = yield _fs2.default.readdir(path);

      for (const dirPath of dirPaths) {
        yield _this7.uploadPath(_path2.default.join(path, dirPath), datPath);
      }
    })();
  }

  mkdir(path) {
    return new Promise(resolve => {
      this.dat.archive.mkdir(path, err => {
        if (err) {
          _logger2.default.error(err.toString());
          _logger2.default.error(`${path}: Failed to create directory in dat archive.`);
          process.exit(1);
        }
        resolve();
      });
    });
  }

  download() {
    var _this8 = this;

    return new Promise(resolve => {
      const abort = setTimeout(() => {
        _logger2.default.error('Failed to download metadata from peer.');
        process.exit(1);
      }, 15000);

      const readRoot = (() => {
        var _ref = _asyncToGenerator(function* () {
          const paths = yield _this8.readdir('/');

          if (paths.length !== 0) {
            clearTimeout(abort);

            for (const path of paths) {
              yield _this8.downloadPath(path);
            }

            resolve();
          } else {
            setTimeout(readRoot, 300);
          }
        });

        return function readRoot() {
          return _ref.apply(this, arguments);
        };
      })();

      readRoot();
    });
  }

  downloadPath(path) {
    var _this9 = this;

    return _asyncToGenerator(function* () {
      const stats = yield _this9.stat(path);

      if (stats.isDirectory()) {
        yield _this9.downloadDir(path, stats);
      } else {
        yield _this9.downloadFile(path, stats);
      }
    })();
  }

  downloadFile(path, stats) {
    var _this10 = this;

    return _asyncToGenerator(function* () {
      // If the file exists and is the same size, assume that it hasn't changed
      // and skip it.
      try {
        const fsStats = yield _fs2.default.lstat(path);
        if (stats.size === fsStats.size) {
          _logger2.default.warn(`${path}: File is identical (not copied).`);
          return;
        }
      } catch (err) {
        // File doesn't exist, do nothing.
      }

      _this10.countPath(path, stats);

      if (_this10.options.dryRun) {
        return;
      }

      const readStream = _this10.dat.archive.createReadStream(path);
      const writeStream = _fs2.default.createWriteStream(path);
      const filesize = stats.size || 1;

      yield (0, _pipeStreams2.default)(readStream, writeStream, filesize, path);
    })();
  }

  downloadDir(path, stats) {
    var _this11 = this;

    return _asyncToGenerator(function* () {
      if (!_this11.options.dryRun) {
        // lstat will throw an error if a path does not exist, so rely on that to
        // know that the dir does not already exist. If the path exists and is not
        // a directory, error.
        try {
          const stats = yield _fs2.default.lstat(path);
          if (!stats.isDirectory()) {
            _logger2.default.error(`${path}: Not a directory.`);
            process.exit(1);
          }
        } catch (err) {
          yield _fs2.default.mkdir(path);
        }
      }

      _this11.countPath(path, stats);

      const dirPaths = yield _this11.readdir(path);

      for (const dirPath of dirPaths) {
        yield _this11.downloadPath(_path2.default.join(path, dirPath));
      }
    })();
  }

  countPath(path, stats) {
    if (path === '.') {
      return;
    }

    this.files += 1;
    this.totalSize += stats.size;

    if (!this.options.dryRun) {
      return;
    }

    if (stats.isFile()) {
      _logger2.default.info(`${(0, _formatSize.formatSize)(stats.size).padEnd(8)} ${path}`);
    } else if (stats.isDirectory()) {
      _logger2.default.info(`${'-'.padEnd(8)} ${path}`);
    }
  }

  printTotal() {
    _logger2.default.info(`\nTotal: ${this.files} files (${(0, _formatSize.formatSize)(this.totalSize)})`);
  }

  downloadPrompt() {
    var _this12 = this;

    return _asyncToGenerator(function* () {
      const answer = yield (0, _prompt2.default)(`\nDownload ${_this12.files} files (${(0, _formatSize.formatSize)(_this12.totalSize)})? [Y/n] `);

      const proceed = ['yes', 'y', ''].includes(answer.trim().toLowerCase());

      if (proceed) {
        _logger2.default.info();
      }

      return proceed;
    })();
  }

  readdir(path) {
    return new Promise((resolve, reject) => {
      this.dat.archive.readdir(path, (() => {
        var _ref2 = _asyncToGenerator(function* (err, paths) {
          if (err) {
            _logger2.default.error(err.toString());
            _logger2.default.error(`${path}: Failed to read from dat archive.`);
            process.exit(1);
          }
          resolve(paths);
        });

        return function (_x, _x2) {
          return _ref2.apply(this, arguments);
        };
      })());
    });
  }

  stat(path) {
    return new Promise((resolve, reject) => {
      this.dat.archive.stat(path, (err, stats) => {
        if (err) {
          _logger2.default.error(err.toString());
          _logger2.default.error(`${path}: Failed to get stats from dat archive.`);
          process.exit(1);
        }
        resolve(stats);
      });
    });
  }

}
exports.default = DatCp;